
;// NTFS $MFT structure & constante
;//------------------------------------

aStandardInformation = 0x10
aAttributeList       = 0x20
aFileName            = 0x30
aObjectId            = 0x40
aSecurityDescriptor  = 0x50
aVolumeName          = 0x60
aVolumeInformation   = 0x70
aData                = 0x80
aIndexRoot           = 0x90
aIndexAllocation     = 0xA0
aBitmap              = 0xB0
aReparsePoint        = 0xC0
aEAInformation       = 0xD0
aEA                  = 0xE0
aPropertySet         = 0xF0
aLoggedUtilityStream = 0x100
aEnd                 = 0xFFFFFFFF

NTFS_FILE_MFT              = 0
NTFS_FILE_MFTMIRR          = 1
NTFS_FILE_LOGFILE          = 2
NTFS_FILE_VOLUME           = 3
NTFS_FILE_ATTRDEF          = 4
NTFS_FILE_ROOT             = 5
NTFS_FILE_BITMAP           = 6
NTFS_FILE_BOOT             = 7
NTFS_FILE_BADCLUS          = 8
NTFS_FILE_QUOTA            = 9
NTFS_FILE_UPCASE           = 10
NTFS_FILE_EXTEND           = 11
NTFS_FILE_FIRST_USER_FILE  = 16
 
NTFS_MFT_MASK              = 0x0000FFFFFFFFFFFF
 
COLLATION_BINARY              = 0x00
COLLATION_FILE_NAME           = 0x01
COLLATION_UNICODE_STRING      = 0x02
COLLATION_NTOFS_ULONG         = 0x10
COLLATION_NTOFS_SID           = 0x11
COLLATION_NTOFS_SECURITY_HASH = 0x12
COLLATION_NTOFS_ULONGS        = 0x13
 
INDEX_ROOT_SMALL = 0x0
INDEX_ROOT_LARGE = 0x1
INDEX_NODE_SMALL = 0x0
INDEX_NODE_LARGE = 0x1
 
NTFS_INDEX_ENTRY_NODE  = 1
NTFS_INDEX_ENTRY_END   = 2
 
NTFS_FILE_NAME_POSIX         = 0
NTFS_FILE_NAME_WIN32         = 1
NTFS_FILE_NAME_DOS           = 2
NTFS_FILE_NAME_WIN32_AND_DOS = 3
 
NTFS_FILE_TYPE_READ_ONLY  = 0x00000001
NTFS_FILE_TYPE_HIDDEN     = 0x00000002
NTFS_FILE_TYPE_SYSTEM     = 0x00000004
NTFS_FILE_TYPE_ARCHIVE    = 0x00000020
NTFS_FILE_TYPE_TEMPORARY  = 0x00000100
NTFS_FILE_TYPE_SPARSE     = 0x00000200
NTFS_FILE_TYPE_REPARSE    = 0x00000400
NTFS_FILE_TYPE_COMPRESSED = 0x00000800
NTFS_FILE_TYPE_OFFLINE    = 0x00001000
NTFS_FILE_TYPE_ENCRYPTED  = 0x00004000
NTFS_FILE_TYPE_DIRECTORY  = 0x10000000

;// NTFS_RECORD_HEADER.Type

NRH_FILE_TYPE  = 0x454C4946  ;// 'FILE'
NRH_INDX_TYPE  = 0x58444E49  ;// 'INDX'

;// Flags in FILE_RECORD_HEADER
 
FRH_IN_DEL     = 0   ;// 
FRH_IN_USE     = 1   ;// 
FRH_DIRECTORY  = 2   ;//  


;//***********************************************************
   
struct BOOT_SECTOR         ;//<---   ! = Ntfs $BOOT
  JumpBootLoader    rb  3      ;// jump  
  SystemId          rb  8      ;//  "NTFS"
  BytePerSector     dw  0
  SectorPerCluster  db  0
  Reserved1         rb  7
  Media             db  0
  Reserved2         dw  0
  SectorPerTrack    dw  0
  Heads             dw  0
  Reserved3         rb  12
  SectorCount       dq  0      ;//    
  MftLcn            dq  0      ;//   $MFT (LCN, Logical_Cluster_Num)
  MftMirrLcn        dq  0      ;//   $MFTMirr
  ClusPerMftEntry   dd  0      ;//    $MFT  
  ClusIndexEntry    dd  0      ;//    Index  
  VolumeSN          dq  0      ;//  
  BpbChecksum       dd  0      ;//   = 0
  ;//  0x0200 = Ntldr.  
ends

struct FILE_RECORD             ;//<-----------  38h (56)
  Magic             db 'FILE'  ;// 
  upsOffset         dw  0      ;//   " "
  uspSize           dw  0      ;//   ..    .
  LSN               dq  0      ;//    $LogFile (LSN, LogicalSN ???)
  SequenceNum       dw  0      ;//    "Record"
  HardLinkCount     dw  0      ;//   
  firstAttrOffset   dw  0      ;//    
  Flags             dw  0      ;// : 0=, 1= , 2=
  fRecordRealSize   dd  0      ;//     "FILE",
  fRecordAllocSize  dd  0      ;//    ..    .
  ReferBaseRecord   dq  0      ;//       (.#2)
  NextAttrId        dw  0      ;// ID .
  Padding1          dw  0      ;// ()
  MftRecordNum      dd  0      ;//     $MFT
  upsNumber         dw  0      ;//  " "
  Padding2          rb  6      ;// ()  
  ;//  .....
ends

struct ATTRIBUTE_HEADER        ;//<-----------  18h (24)
  Type              dd  0      ;//  ,
  AttrLen           dd  0      ;//  ..  .
  ResidentFlag      db  0      ;//  : 0=, 1=
  AttrNameLen       db  0      ;//     
  AttrNameOffset    dw  0      ;//   
  Flags             dw  0      ;// : 0=, 1=
  AttrId            dw  0      ;// ID  (  )
  AttrBodyLen       dd  0      ;//   
  AttrOffset        dw  0      ;//   
  IndexedFlag       db  0      ;//  
  Padding           db  0      ;// ()
  ;//  ......
ends

struct ATTR_STD_INFO_10        ;//<-----------  60h (96)
  Header            ATTRIBUTE_HEADER
  fCreationTime     FILETIME   ;//  
  fAlteredTime      FILETIME   ;//  
  mftChangedTime    FILETIME   ;//   MFT
  fReadTime         FILETIME   ;//   
  fPermissions      dd  0      ;//   (.)
  MaxVerNum         dd  0      ;// . 
  VerNum            dd  0      ;// 
  ClassId           dd  0      ;// ID  ???
  OwnerId           dd  0      ;// ID 
  SecurityId        dd  0      ;// ID 
  QuotaCherged      dq  0      ;//   
  USN               dq  0      ;//     "Update Sequence Number"
ends

struct ATTR_FILE_NAME_30       ;//<-----------      
  Header            ATTRIBUTE_HEADER
  ParentDir         dq  0      ;//          
  fCreationTime     FILETIME   ;//  
  fAlteredTime      FILETIME   ;//  
  mftChangedTime    FILETIME   ;//   MFT
  fReadTime         FILETIME   ;//   
  AllocaFileSize    dq  0      ;//   
  RealFileSize      dq  0      ;//   
  FileFlags         dd  0      ;//   (dir=1000.0000h,    $MFT)
  Reserved          dd  0      ;// 
  FileNameLen       db  0      ;//   
  Padding           db  0      ;// ()
  FileName          db  0      ;// Unicode  
ends

struct ATTR_INDEX_ROOT_90_ROOT ;//<-----------  28h (40)
  Header            ATTRIBUTE_HEADER
  AttrType          dd  0      ;//    : 30 = $FILE_NAME
  CollationRule     dd  0      ;//   : 1=  .
  IndexAllocSize    dd  0      ;//     (  )
  ClustersPerIndex  db  0      ;//    
  Padding           rb  3      ;// ()
ends

struct ATTR_INDEX_ROOT_90_HEADER   ;//<-----------  10h (16)
  FirstIndexOffset  dd  0      ;//    
  SizeIndexEntries  dd  0      ;//    
  AllocSize         dd  0      ;//   
  Flags             db  0      ;// 
  Padding           rb  3      ;// ()
ends

;// List of Common Indexes
;// Name   Index Of        Used By
;// ----------------------------------
;// $I30   Filenames       Directories
;// $SDH   Security Desc   $Secure
;// $SII   Security Ids    $Secure
;// $O     Object Ids      $ObjId
;// $O     Owner Ids       $Quota
;// $Q     Quotas          $Quota
;// $R     Reparse Points  $Reparse

struct INDEX_HEADER        ;//<-----------  
  Magic             db 'INDX'
  upsOffset         dw  0      ;//   " "
  upsSize           dw  0      ;//  ..    .
  LSN               dq  0      ;//    $LogFile
  vcnIndx           dq  0      ;// "Virtual Cluster Number"  Index Allocation
  EntriesOffset     dd  0      ;//   ,
  EntriesSize       dd  0      ;//  ..    ,
  AllocaSize        dd  0      ;//    .. +  .
  NotLeafNode       db  0      ;// 1 =     
  Padding           rb  3      ;// ()
  UpdateSequence    dw  0      ;//  UPS
  ;//   UpdateSequenceArray
ends  

if 0
struct INDEX_RECORD             ;//<----------- 
  upsOffset         dw  0      ;//   " "

ends
end if















